/************************************************************************
* Name:         Sampled Material Import
* Description:  This module reads a text file containing the n-k diagram
                in the following format
                wavelength [Tab] n [Tab] k
                and generates a material in the user defined material catalog.
* Authors:      Roberto Knoth (LightTrans)
* Version:      1.1 (January 31st, 2017)
* License:      Some Rights reserved, CC-BY 3.0
*************************************************************************/

using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;

using VirtualLab.Programming;
using VirtualLabAPI.Core.BasicFunctions;
using VirtualLabAPI.Core.Common;
using VirtualLabAPI.Core.DataVisualization;
using VirtualLabAPI.Core.FieldRepresentations;
using VirtualLabAPI.Core.GeometryDescription;
using VirtualLabAPI.Core.LightPath;
using VirtualLabAPI.Core.Materials;
using VirtualLabAPI.Core.Modules;
using VirtualLabAPI.Core.Numerics;
using VirtualLabAPI.Core.OpticalSystems;
using VirtualLabAPI.Core.OpticalSystems.Media;
using VirtualLabAPI.Core.Propagation;

namespace OwnCode {
    public class VLModule : IVLModule {
        public void Run() {
            
            string materialToSave = "Testmaterial";
            string categoryOfMaterial = "My Materials";
            string fileName_n_k_Data = @"D:\_VL_temp\AG2.txt";
            double wLfactor = 1E-6; //sets the wavelength magnitude of the given data
            
            // generate material object
            StandardMaterial material2Process = new StandardMaterial();
            
            // import n-k data - please verify, that the format is wavelength \t n \t k
            ComplexField imported_n_k_Data = ImportExport.ImportASCII(fileName_n_k_Data, '\t', ',', PrecisionMode.Float);
            
            #region generate data container for material object
            ComplexFieldDerivative1D refrIndexData = new ComplexFieldDerivative1D(2);
            ComplexFieldDerivative1D absorpCoeffsData = new ComplexFieldDerivative1D(2);
            CFieldDerivative1DReal wavelength = new CFieldDerivative1DReal(2);
            if (imported_n_k_Data.SamplingPoints.X == 3){
                wavelength = new CFieldDerivative1DReal(imported_n_k_Data.SamplingPoints.Y);
                refrIndexData = new ComplexFieldDerivative1D(imported_n_k_Data.SamplingPoints.Y);
                absorpCoeffsData = new ComplexFieldDerivative1D(imported_n_k_Data.SamplingPoints.Y);
                
                for (long runDataPoints = 0; runDataPoints < imported_n_k_Data.SamplingPoints.Y; runDataPoints++) {
                    wavelength[runDataPoints] = imported_n_k_Data[0, runDataPoints].Re * wLfactor;
                    refrIndexData[runDataPoints] = imported_n_k_Data[1, runDataPoints].Re;
                    absorpCoeffsData[runDataPoints] = MaterialSupportFunctions.AbsorptionCoefficientFromImaginaryRefractiveIndex(imported_n_k_Data[2,runDataPoints].Re,wavelength[runDataPoints]);
                }
                
                refrIndexData.IsComplex = false;
                absorpCoeffsData.IsComplex = false;
            }
            else if (imported_n_k_Data.SamplingPoints.Y == 3) {
                wavelength = new CFieldDerivative1DReal(imported_n_k_Data.SamplingPoints.X);
                refrIndexData = new ComplexFieldDerivative1D(imported_n_k_Data.SamplingPoints.X);
                absorpCoeffsData = new ComplexFieldDerivative1D(imported_n_k_Data.SamplingPoints.X);
                
                for (long runDataPoints = 0; runDataPoints < imported_n_k_Data.SamplingPoints.X; runDataPoints++) {
                    wavelength[runDataPoints] = imported_n_k_Data[runDataPoints, 0].Re * wLfactor;
                    refrIndexData[runDataPoints] = imported_n_k_Data[runDataPoints, 1].Re;
                    absorpCoeffsData[runDataPoints] = MaterialSupportFunctions.AbsorptionCoefficientFromImaginaryRefractiveIndex(imported_n_k_Data[runDataPoints, 2].Re,wavelength[runDataPoints]);
                }        
                
                refrIndexData.IsComplex = false;
                absorpCoeffsData.IsComplex = false;
            } 
            else {
                Globals.DataDisplay.LogError("Format of imported Data not supported!");                
            }
            
            // generate DataArray1D containgin wavelength and refractive index
            DataArray1D refrIndexDataArray1D = new DataArray1D(
                new ComplexField1DArray(new ComplexFieldDerivative1D[]{refrIndexData}),
                new PhysicalProperty[]{PhysicalProperty.NoUnit},
                new String[]{"n"},
                wavelength,
                PhysicalProperty.Length,
                "wavelength",
                1.5 * wavelength[wavelength.SamplingPointsOneD - 1] - 0.5 * wavelength[wavelength.SamplingPointsOneD - 2]);
            
            // generate DataArray1D containgin wavelength and absorption coefficient
            DataArray1D absorpCoeffsDataArray1D = new DataArray1D(
                new ComplexField1DArray(new ComplexFieldDerivative1D[]{absorpCoeffsData}),
                new PhysicalProperty[]{PhysicalProperty.NoUnit},
                new String[]{"alpha"},
                wavelength,
                PhysicalProperty.Length,
                "wavelength",
                1.5 * wavelength[wavelength.SamplingPointsOneD - 1] - 0.5 * wavelength[wavelength.SamplingPointsOneD - 2]);
            #endregion
                      
            // some output
            VL_GUI.ShowDocument(refrIndexDataArray1D, "Refractive Indices");
            VL_GUI.ShowDocument(absorpCoeffsDataArray1D, "Absorption Coefficients");
            
            // fill material with n-k data
            material2Process.SampledAbsorptionCoeff = absorpCoeffsDataArray1D;
            material2Process.SampledRefractiveIndex = refrIndexDataArray1D;
            
            // save material to user defined material catalog
            SaveMaterialToUserDefinedMaterialCatalog(material2Process, materialToSave, categoryOfMaterial);
        }
            
        // load material from material catalog
        private Material LoadMaterialFromCatalog(string nameOfMaterialInCatalog, CatalogDefinitionType catalogDefType) {
            // load catalog from default path
            MaterialsCatalog materialsCatalog = MaterialsCatalog.LoadMaterialsCatalogFromDefaultPath(catalogDefType, true);
            
            if (materialsCatalog == null) {
                Globals.DataDisplay.LogError("Material catalog does not exist!");
                return null;
            }
        
            // search for material and load
            if(materialsCatalog.ContainsEntryWithName(nameOfMaterialInCatalog)) {
                return materialsCatalog[nameOfMaterialInCatalog];
            }
            else {
                Globals.DataDisplay.LogError("Material " + nameOfMaterialInCatalog + " does not exist in Catalog.");
                return null;
            }
        }
        
        // save material to user defined catalog
        private void SaveMaterialToUserDefinedMaterialCatalog(Material material2Save, string nameOfMaterial, string categoryOfMaterial) {
            // load catalog from default path
            MaterialsCatalog materialsCatalog = MaterialsCatalog.LoadMaterialsCatalogFromDefaultPath(CatalogDefinitionType.UserDefined, true);
              
            // check if name already exists in catalog
            if(!materialsCatalog.ContainsEntryWithName(nameOfMaterial)) {
                // set name of material
                material2Save.Name = nameOfMaterial;
                List<string> category = new List<string>();
                category.Add(categoryOfMaterial);
                material2Save.Categories = category;
                
                // add material to catalog
                materialsCatalog.Add(material2Save);
                
                // save catalog
                materialsCatalog.Save(materialsCatalog.FileNameOfUserDefinedCatalog);
            }
            else {
                Globals.DataDisplay.LogError("Name of Material already exists within catalog!");
            }
        }
    }
}